/*
 * Copyright (C) 2011 Thomas Akehurst
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * This is a modification of HandlebarsJsonPathHelper.
 */
package com.trafficparrot.sdk.handlebars;

import com.github.jknack.handlebars.Options;
import com.github.tomakehurst.wiremock.common.Json;
import com.github.tomakehurst.wiremock.extension.responsetemplating.helpers.HandlebarsJsonPathHelper;
import com.jayway.jsonpath.InvalidJsonException;
import com.jayway.jsonpath.JsonPath;
import com.jayway.jsonpath.JsonPathException;
import com.jayway.jsonpath.PathNotFoundException;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

// TODO(4/5) push to wiremock or handlebars java?

/**
 * This class mimics the behaviour of {@link HandlebarsJsonPathHelper} in order to extract a list of nodes.
 */
public class JsonPathList extends TrafficParrotHelper<String> {

    @Override
    public Object doApply(final String inputJson, final Options options) throws IOException {
        if (inputJson == null) {
            return "{{ Warning: input JSON parameter is missing }}";
        }

        if (options == null || options.param(0, null) == null) {
            return "{{ Warning: JSONPath parameter is missing }}";
        }

        String jsonPath = options.param(0);
        try {
            Object result = JsonPath.read(inputJson, jsonPath);
            if (!(result instanceof List)) {
                return "{{ Warning: '" + jsonPath + "' is not a list }}";
            }
            List<?> resultList = (List<?>) result;
            List<String> resultListJson = new ArrayList<>(resultList.size());
            for (Object object : resultList) {
                resultListJson.add(Json.write(object));
            }
            return resultListJson;
        } catch (InvalidJsonException e) {
            return "{{ Warning: '" + inputJson + "' is not valid JSON }}";
        }  catch (PathNotFoundException e) {
            return "{{ Warning: '" + jsonPath + "' was not found }}";
        } catch (JsonPathException e) {
            return "{{ Warning: '" + jsonPath + "' is not a valid JSONPath expression }}";
        }
    }
}
