package com.trafficparrot.sdk.ibmmq;

import com.github.jknack.handlebars.Handlebars;
import com.github.jknack.handlebars.HandlebarsException;
import com.github.jknack.handlebars.Template;
import com.trafficparrot.sdk.jms.TpLogger;

import java.io.IOException;

import static com.trafficparrot.sdk.handlebars.HandlebarsFactory.handlebars;
import static java.lang.String.format;
import static java.util.Collections.emptyMap;

public class IbmMqHandlebarsResponseTransformer {
    private final Handlebars handlebars;
    private final boolean enabled;
    private final TpLogger logger;

    public IbmMqHandlebarsResponseTransformer(Handlebars handlebars, boolean enabled, TpLogger logger) {
        this.handlebars = handlebars;
        this.enabled = enabled;
        this.logger = logger;
    }

    public static IbmMqHandlebarsResponseTransformer handlebarsTransformer(String commaSeparatedHandlebarHelpers, Boolean handlebarsEnabled, TpLogger logger) {
        if (handlebarsEnabled) {
            return defineHelpers(logger, handlebars(commaSeparatedHandlebarHelpers));
        }
        return handlebarsAreDisabled(logger);
    }

    public static IbmMqHandlebarsResponseTransformer handlebarsAreDisabled(TpLogger logger) {
        return new IbmMqHandlebarsResponseTransformer(null, false, logger);
    }

    private static IbmMqHandlebarsResponseTransformer defineHelpers(TpLogger logger, Handlebars handlebars) {
        return new IbmMqHandlebarsResponseTransformer(handlebars, true, logger);
    }

    public IbmMqResponse transform(IbmMqResponse response) {
        if (enabled) {
            try {
                Template responseTemplate = handlebars.compileInline(response.text);
                String responseTextWithTemplateApplied = responseTemplate.apply(emptyMap());
                return new IbmMqResponse(response.destination, response.ibmMqResponseTransformerClassName, responseTextWithTemplateApplied);
            } catch (HandlebarsException | IOException e) {
                String errorMessage = format("Problem while applying handlebars: \n\n%s\n\nOriginal text: \n%s", e.getMessage(), response.text);
                logger.error(errorMessage, e);
                return new IbmMqResponse(response.destination, errorMessage + ". \n\nSee logs for more details.");
            }
        } else {
            return response;
        }
    }

    // TODO(4/5): uncomment when we wire this class in properly (moving to trafficparrot-sdk-ibm-mq or whatever)
//    private Map<String, ?> requestModel(MQMessage requestMessage) throws JMSException {
//        String text = IbmMqUtils.readMessage(requestMessage);
//        return singletonMap("request", singletonMap("body", text));
//    }
}
