package com.trafficparrot.sdk.ibmmq;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.collect.ComparisonChain;
import com.trafficparrot.sdk.jms.Destination;

import static org.apache.commons.lang3.builder.ToStringBuilder.reflectionToString;
import static org.apache.commons.lang3.builder.ToStringStyle.SHORT_PREFIX_STYLE;

public class IbmMqResponse implements Comparable<IbmMqResponse> {
    public static final String NO_TRANSFORMER = "NO_TRANSFORMER";

    public final Destination destination;
    public final String ibmMqResponseTransformerClassName;
    public final String text;

    public IbmMqResponse(@JsonProperty("destination") Destination destination,
                         @JsonProperty("ibmMqResponseTransformerClassName") String ibmMqResponseTransformerClassName,
                         @JsonProperty("text") String text) {
        this.destination = destination;
        this.ibmMqResponseTransformerClassName = ibmMqResponseTransformerClassName;
        this.text = text;
    }

    public IbmMqResponse(Destination destination, String text) {
        this.destination = destination;
        this.ibmMqResponseTransformerClassName = NO_TRANSFORMER;
        this.text = text;
    }

    @Override
    @JsonIgnore
    public String toString() {
        return reflectionToString(this, SHORT_PREFIX_STYLE);
    }

    @Override
    public int compareTo(IbmMqResponse other) {
        return ComparisonChain.start()
                .compare(destination, other.destination)
                .compare(text, other.text)
                .result();
    }
}
