package com.trafficparrot.sdk.jms;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.github.tomakehurst.wiremock.matching.ContentPattern;
import com.google.common.collect.ComparisonChain;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;

import static com.trafficparrot.sdk.jms.JmsMessageType.jmsMessageType;
import static com.trafficparrot.sdk.jms.MessageBodyType.TEXT;
import static com.trafficparrot.sdk.jms.NamedValueMatcherComparator.NAMED_VALUE_MATCHER_COMPARATOR;
import static org.apache.commons.lang3.ObjectUtils.firstNonNull;
import static org.apache.commons.lang3.builder.ToStringBuilder.reflectionToString;
import static org.apache.commons.lang3.builder.ToStringStyle.SHORT_PREFIX_STYLE;

public class JmsRequestMatcher implements Comparable<JmsRequestMatcher> {
    public final Destination destination;
    public final ContentPattern<?> bodyMatcher;
    public final MessageBodyType bodyType;
    public final String jmsMessageType;

    public JmsRequestMatcher(@JsonProperty("destination") Destination destination,
                             @JsonProperty("bodyMatcher") ContentPattern<?> bodyMatcher,
                             @JsonProperty("bodyType") MessageBodyType bodyType,
                             @JsonProperty("jmsMessageType") String jmsMessageType) {
        this.destination = destination;
        this.bodyMatcher = bodyMatcher;
        this.bodyType = firstNonNull(bodyType, TEXT);
        this.jmsMessageType = firstNonNull(jmsMessageType, jmsMessageType(this.bodyType));
    }

    public JmsRequestMatcher(Destination destination, ContentPattern<?> bodyMatcher) {
        this(destination, bodyMatcher, MessageBodyType.TEXT);
    }

    public JmsRequestMatcher(Destination destination, ContentPattern<?> bodyMatcher, MessageBodyType bodyType) {
        this(destination, bodyMatcher, bodyType, jmsMessageType(bodyType));
    }

    @Override
    @JsonIgnore
    public String toString() {
        return reflectionToString(this, SHORT_PREFIX_STYLE);
    }

    @Override
    public int hashCode() {
        return HashCodeBuilder.reflectionHashCode(this);
    }

    @Override
    public boolean equals(Object obj) {
        return EqualsBuilder.reflectionEquals(this, obj);
    }

    @Override
    public int compareTo(JmsRequestMatcher other) {
        return ComparisonChain.start()
                .compare(destination, other.destination)
                .compare(bodyMatcher, other.bodyMatcher, NAMED_VALUE_MATCHER_COMPARATOR)
                .result();
    }
}
