package com.trafficparrot.sdk.jms;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

import java.lang.reflect.InvocationHandler;
import java.util.Objects;

import static com.trafficparrot.sdk.messaging.ObjectSerializedRepresentation.OBJECT_SERIALIZED_REPRESENTATION;
import static java.lang.String.format;
import static java.lang.reflect.Proxy.getInvocationHandler;
import static java.lang.reflect.Proxy.isProxyClass;

@JsonSerialize(using = MessageBodyTypeSerializer.class)
@JsonDeserialize(using = MessageBodyTypeDeserializer.class)
public class MessageBodyType {
    public static MessageBodyType TEXT = new MessageBodyType("TEXT");
    public static MessageBodyType BYTES = new MessageBodyType("BYTES");
    public static MessageBodyType PRINTABLE_CHARACTERS = new MessageBodyType("PRINTABLE_CHARACTERS");

    public final String value;

    public MessageBodyType(String value) {
        this.value = value;
    }

    public static MessageBodyType javaClassType(Object object) {
        if (isProxyClass(object.getClass())) {
            InvocationHandler invocationHandler = getInvocationHandler(object);
            return new MessageBodyType(OBJECT_SERIALIZED_REPRESENTATION.externalClassName(invocationHandler.getClass().getName()));
        } else {
            return new MessageBodyType(OBJECT_SERIALIZED_REPRESENTATION.externalClassName(object.getClass().getName()));
        }
    }

    @Override
    @JsonIgnore
    public String toString() {
        return value;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        MessageBodyType that = (MessageBodyType) o;
        return Objects.equals(value, that.value);
    }

    @Override
    public int hashCode() {
        return Objects.hash(value);
    }

    @JsonIgnore
    public boolean isJavaObject() {
        if (TEXT.equals(this) || BYTES.equals(this) || PRINTABLE_CHARACTERS.equals(this)) {
            return false;
        }
        try {
            Class.forName(OBJECT_SERIALIZED_REPRESENTATION.internalClassName(value));
            return true;
        } catch (ClassNotFoundException e) {
            return false;
        }
    }

    @JsonIgnore
    public Class<?> javaClass() {
        try {
            return Class.forName(OBJECT_SERIALIZED_REPRESENTATION.internalClassName(value));
        } catch (ClassNotFoundException e) {
            throw new IllegalStateException(format("The class '%s' is not on the Traffic Parrot classpath", value), e);
        }
    }
}
