package com.trafficparrot.sdk.messaging;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;

import java.io.IOException;
import java.lang.reflect.Proxy;

import static java.lang.reflect.Proxy.getInvocationHandler;

public class ObjectJsonRepresentation {

    public static String objectToJson(Object object) throws JsonProcessingException {
        if (object instanceof Proxy) {
            // this will be a com.sun.proxy and so it is more useful to write the invocation handler not the proxy
            return objectMapper().writeValueAsString(getInvocationHandler(object));
        } else {
            return objectMapper().writeValueAsString(object);
        }
    }

    public static <T> T jsonToObject(String text, Class<T> type) throws IOException {
        return objectMapper().readValue(text, type);
    }

    private static ObjectMapper objectMapper() {
        ObjectMapper objectMapper = new ObjectMapper();
        objectMapper.enable(SerializationFeature.INDENT_OUTPUT);
        objectMapper.disable(SerializationFeature.FAIL_ON_EMPTY_BEANS);
        objectMapper.setVisibility(PropertyAccessor.ALL, JsonAutoDetect.Visibility.NONE);
        objectMapper.setVisibility(PropertyAccessor.FIELD, JsonAutoDetect.Visibility.ANY);
        return objectMapper;
    }
}
