package com.trafficparrot.sdk.messaging;

import com.google.common.annotations.VisibleForTesting;

import static java.lang.Character.isLetterOrDigit;
import static java.lang.Character.isWhitespace;
import static java.nio.charset.StandardCharsets.UTF_8;
import static org.apache.commons.lang3.CharUtils.isAsciiPrintable;

public class PrintableCharactersRepresentation {

    private static final double MOSTLY_PRINTABLE_THRESHOLD = 0.95;

    public static String encodePrintableCharacters(byte[] bytes) {
        return new String(bytes, UTF_8);
    }

    public static byte[] decodePrintableCharacters(String text) {
        return text.getBytes(UTF_8);
    }

    public static boolean isMostlyPrintableCharacters(byte[] bytes) {
        return (double) numberOfPrintableCharacters(bytes) / bytes.length > MOSTLY_PRINTABLE_THRESHOLD;
    }

    @VisibleForTesting
    static int numberOfPrintableCharacters(byte[] bytes) {
        String string = encodePrintableCharacters(bytes);
        int count = 0;
        for (int i = string.length() - 1; i >= 0; i--) {
            char character = string.charAt(i);
            if (isAsciiPrintable(character) || isLetterOrDigit(character) || isWhitespace(character)) {
                count++;
            }
        }
        return count;
    }
}
